Stan Math Library: stan/math/prim/scal/prob/poisson_log_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

poisson_log_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_POISSON_LOG_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_POISSON_LOG_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_less.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/multiply_log.hpp>
13 #include <stan/math/prim/scal/fun/gamma_q.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <stan/math/prim/scal/meta/include_summand.hpp>
16 #include <stan/math/prim/scal/meta/VectorView.hpp>
17 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
18 #include <boost/math/special_functions/fpclassify.hpp>
19 #include <boost/random/poisson_distribution.hpp>
20 #include <boost/random/variate_generator.hpp>
21 #include <cmath>
22 #include <limits>
23
24 namespace stan {
25
26 namespace math {
27
28 // PoissonLog(n|alpha) [n >= 0] = Poisson(n|exp(alpha))
29 template <bool propto,
30 typename T_n, typename T_log_rate>
31 typename return_type<T_log_rate>::type
32 poisson_log_log(const T_n& n, const T_log_rate& alpha) {
33 typedef typename stan::partials_return_type<T_n, T_log_rate>::type
34 T_partials_return;
35
36 static const char* function("stan::math::poisson_log_log");
37
38 using boost::math::lgamma;
39 using stan::math::check_not_nan;
40 using stan::math::check_nonnegative;
41 using stan::math::value_of;
42 using stan::math::check_consistent_sizes;
43 using stan::math::include_summand;
44 using std::exp;
45 using std::exp;
46
47 // check if any vectors are zero length
48 if (!(stan::length(n)
49 && stan::length(alpha)))
50 return 0.0;
51
52 // set up return value accumulator
53 T_partials_return logp(0.0);
54
55 // validate args
56 check_nonnegative(function, "Random variable", n);
57 check_not_nan(function, "Log rate parameter", alpha);
58 check_consistent_sizes(function,
59 "Random variable", n,
60 "Log rate parameter", alpha);
61
62 // check if no variables are involved and prop-to
63 if (!include_summand<propto, T_log_rate>::value)
64 return 0.0;
65
66 // set up expression templates wrapping scalars/vecs into vector views
67 VectorView<const T_n> n_vec(n);
68 VectorView<const T_log_rate> alpha_vec(alpha);
69 size_t size = max_size(n, alpha);
70
71 // FIXME: first loop size of alpha_vec, second loop if-ed for size==1
72 for (size_t i = 0; i < size; i++)
73 if (std::numeric_limits<double>::infinity() == alpha_vec[i])
74 return LOG_ZERO;
75 for (size_t i = 0; i < size; i++)
76 if (-std::numeric_limits<double>::infinity() == alpha_vec[i]
77 && n_vec[i] != 0)
78 return LOG_ZERO;
79
80 // return accumulator with gradients
81 OperandsAndPartials<T_log_rate> operands_and_partials(alpha);
82
83 // FIXME: cache value_of for alpha_vec? faster if only one?
84 VectorBuilder<include_summand<propto, T_log_rate>::value,
85 T_partials_return, T_log_rate>
86 exp_alpha(length(alpha));
87 for (size_t i = 0; i < length(alpha); i++)
88 if (include_summand<propto, T_log_rate>::value)
89 exp_alpha[i] = exp(value_of(alpha_vec[i]));
90
91 using stan::math::multiply_log;
92 for (size_t i = 0; i < size; i++) {
93 if (!(alpha_vec[i] == -std::numeric_limits<double>::infinity()
94 && n_vec[i] == 0)) {
95 if (include_summand<propto>::value)
96 logp -= lgamma(n_vec[i] + 1.0);
97 if (include_summand<propto, T_log_rate>::value)
98 logp += n_vec[i] * value_of(alpha_vec[i]) - exp_alpha[i];
99 }
100
101 // gradients
102 if (!is_constant_struct<T_log_rate>::value)
103 operands_and_partials.d_x1[i] += n_vec[i] - exp_alpha[i];
104 }
105 return operands_and_partials.value(logp);
106 }
107
108 template <typename T_n,
109 typename T_log_rate>
110 inline
111 typename return_type<T_log_rate>::type
112 poisson_log_log(const T_n& n, const T_log_rate& alpha) {
113 return poisson_log_log<false>(n, alpha);
114 }
115 }
116 }
117 #endif
check_not_nan.hpp
stan::math::poisson_log_log
return_type< T_log_rate >::type poisson_log_log(const T_n &n, const T_log_rate &alpha)
Definition: poisson_log_log.hpp:32
stan::math::lgamma
fvar< T > lgamma(const fvar< T > &x)
Definition: lgamma.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::LOG_ZERO
const double LOG_ZERO
Definition: constants.hpp:175
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
gamma_q.hpp
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::multiply_log
fvar< T > multiply_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: multiply_log.hpp:15
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::size
int size(const std::vector< T > &x)
Return the size of the specified standard vector.
Definition: size.hpp:17
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_less.hpp
check_consistent_sizes.hpp
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.